==> hakk_builder-0.2.1.tar.gz: test/basic_test.dart <==
library hakkit.hakk.builder.test.basic;

import 'package:unittest/unittest.dart';
import 'package:hakk_builder/basic.dart' hide equals;

void basic_tests(){
  group('basic -', (){
    final d = new Debugger();
    
    test('missing header throws', (){
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('no system block throws', (){
      hakk('test');
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('core block initializes implicitly', (){
      _top();
      output('hi');
      build();
      expect(d.currentCodeBlock, equals('core'));
    });
    
    test('core block initializes explicitely', (){
      _top();
      codeEntry();
      output('hi');
      expect(d.currentCodeBlock, equals('core'));
      expect(d.totalCodeBlocks, equals(1));
    });
    
    test('core block works out of sequence', (){
      _top();
      Block.instructions('foo', 0, 1);
      codeBlock('foo');
        output('hi');
      
      expect(d.currentCodeBlock, equals('foo'));
      expect(d.totalCodeBlocks, equals(1));
        
      codeEntry();
        output('hi');

      expect(d.currentCodeBlock, equals('core'));
      expect(d.totalCodeBlocks, equals(2));
    });
    
    test('FOR without NEXT throws', (){
      _top();
      FOR(core0 << 0, core0 < 10, () => inc(core0));
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('WHILE without ENDWHILE throws', (){
      _top();
      WHILE(true);
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('IF without ENDIF throws', (){
      _top();
      IF(true);
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('NEXT without FOR throws', (){
      _top();
      expect(() => NEXT(), throwsA(new isInstanceOf<String>()));
    });
    
    test('ENDWHILE without WHILE throws', (){
      _top();
      expect(() => ENDWHILE(), throwsA(new isInstanceOf<String>()));
    });
    
    test('ENDIF without IF throws', (){
      _top();
      expect(() => ENDIF(), throwsA(new isInstanceOf<String>()));
    });
    
    test('MMU has 0 allocated at beginning of compilation.', (){
      _top();
      build();
      expect(d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after expression assignment.', (){
      _top();
      core0 << val(5) + 5;
      expect(d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after FOR/NEXT', (){
      _top();
      FOR(core0 << 0, core0 < 10, () => inc(core0));
        core1 << core0 + val(5) * val(2);
      NEXT();
      
      build();
      expect(d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after nested FOR/NEXT', (){
      _top();
      
      FOR(core0 << 0, core0 < 10, () => inc(core0));
        FOR(core1 << 0, core1 < 10, () => inc(core1));
          core2 << core0 + core1 * val(2);
        NEXT();
      NEXT();
      
      build();
      expect(d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after WHILE loop', (){
      _top();
      
      core0 << 0;
      WHILE(core0 < 10);
        inc(core0);
      ENDWHILE();
      
      build();
      expect(d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after nested WHILE loop', (){
      _top();
      
      core0 << 0;
      WHILE(core0 < 10);
        inc(core0);
        core1 << 0;
        WHILE(core1 < 10);
          inc(core1);
        ENDWHILE();
      ENDWHILE();
      
      build();
      expect(d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after IF/ENDIF', (){
      _top();
      
      core0 << 0;
      IF((core0 * val(5)) < val(10));
        
      ENDIF();
      
      build();
    });
  });
  
}

void _top(){
  hakk('test');
  Block.system(0, -1);
}

==> hakk_builder-0.2.2.tar.gz: test/basic_test.dart <==
library hakkit.hakk.builder.test.basic;

import 'package:unittest/unittest.dart';
import 'package:hakk_builder/basic.dart' hide equals;

final _d = new Debugger();

/// Executes unit tests against the basic API
void basic_tests(){
  group('basic -', (){    
    test('missing header throws', (){
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('no system block throws', (){
      hakk('test');
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('core block initializes implicitly', (){
      _top();
      output('hi');
      build();
      expect(_d.currentCodeBlock, equals('core'));
    });
    
    test('core block initializes explicitely', (){
      _top();
      codeEntry();
      output('hi');
      expect(_d.currentCodeBlock, equals('core'));
      expect(_d.totalCodeBlocks, equals(1));
    });
    
    test('core block works out of sequence', (){
      _top();
      Block.instructions('foo', 0, 1);
      codeBlock('foo');
        output('hi');
      
      expect(_d.currentCodeBlock, equals('foo'));
      expect(_d.totalCodeBlocks, equals(1));
        
      codeEntry();
        output('hi');

      expect(_d.currentCodeBlock, equals('core'));
      expect(_d.totalCodeBlocks, equals(2));
    });
    
    test('FOR without NEXT throws', (){
      _top();
      FOR(core0 << 0, core0 < 10, () => inc(core0));
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('WHILE without ENDWHILE throws', (){
      _top();
      WHILE(true);
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('IF without ENDIF throws', (){
      _top();
      IF(true);
      expect(() => build(), throwsA(new isInstanceOf<String>()));
    });
    
    test('NEXT without FOR throws', (){
      _top();
      expect(() => NEXT(), throwsA(new isInstanceOf<String>()));
    });
    
    test('ENDWHILE without WHILE throws', (){
      _top();
      expect(() => ENDWHILE(), throwsA(new isInstanceOf<String>()));
    });
    
    test('ENDIF without IF throws', (){
      _top();
      expect(() => ENDIF(), throwsA(new isInstanceOf<String>()));
    });
    
    test('MMU has 0 allocated at beginning of compilation.', (){
      _top();
      build();
      expect(_d.MMUAllocated, equals(0));
    });
    
    test('MMU no leaks after expression assignment.', (){
      _top();
      core0 << val(5) + 5;
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after FOR/NEXT', (){
      _top();
      FOR(core0 << 0, core0 < 10, () => inc(core0));
        core1 << core0 + val(5) * val(2);
      NEXT();
      
      build();
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after nested FOR/NEXT', (){
      _top();
      
      FOR(core0 << 0, core0 < 10, () => inc(core0));
        FOR(core1 << 0, core1 < 10, () => inc(core1));
          core2 << core0 + core1 * val(2);
        NEXT();
      NEXT();
      
      build();
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after WHILE loop', (){
      _top();
      
      core0 << 0;
      WHILE(core0 < 10);
        inc(core0);
      ENDWHILE();
      
      build();
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after nested WHILE loop', (){
      _top();
      
      core0 << 0;
      WHILE(core0 < 10);
        inc(core0);
        core1 << 0;
        WHILE(core1 < 10);
          inc(core1);
        ENDWHILE();
      ENDWHILE();
      
      build();
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after IF/ENDIF', (){
      _top();
      
      core0 << 1;
      IF((core0 * val(5)) < val(10));
        core1 << core0 * 2;
      ENDIF();
      
      build();
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after subroutine call', (){
      _top();
      
      SUB('do_eet', [core14, core15]);
        core13 << core14 * core15;
      RETURN(core13 * 5);
      
      FOR(core0 << 0, core0 < 10, () => inc(core0));
        core1 << GOSUB('do_eet', [core0, 3]);
        output(core1);
      NEXT();
      
      build();
      expect(_mmuBalance(), equals(0));
    });
    
    test('MMU no leaks after tailing anonymous expression', (){
      _top();
           
      FOR(core0 << 0, core0 < 10, () => inc(core0));
        core1 << GOSUB('do_eet', [core0, 3]);
        output(core1);
      NEXT();
      
      SUB('do_eet', [core14, core15]);
        core13 << core14 * core15;
        
      // tailing anonymous expression
      RETURN(core13 * 5);
      
      expect(_mmuBalance(), equals(1));
      build();
      expect(_mmuBalance(), equals(0));
    });
  });
  
  //runtime tests will be handled in engine/test/basic_api_runtime_test.dart
}

//should always = 0 after a build()
int _mmuBalance() =>  _d.MMUTotal - (_d.MMUAvailable + _d.totalMarkedForGC);

void _top(){
  hakk('test');
  Block.system(0, -1);
}
